﻿Option Explicit On 
Option Strict On
Imports System
Imports Microsoft.Office.Interop.InfoPath.SemiTrust
Imports Microsoft.VisualBasic
Imports System.Xml

' Office integration attribute. Identifies the startup class for the form. Do not
' modify.
<Assembly: System.ComponentModel.DescriptionAttribute("InfoPathStartupClass, Version=1.0, Class=NWOrdersWSProject.NWOrdersWSProject")> 

Namespace NWOrdersWSProject
	' The namespace prefixes defined in this attribute must remain synchronized with
	' those in the form definition file (.xsf).
	<InfoPathNamespace("xmlns:xsf=""http://schemas.microsoft.com/office/infopath/2003/solutionDefinition"" xmlns:msxsl=""urn:schemas-microsoft-com:xslt"" xmlns:xd=""http://schemas.microsoft.com/office/infopath/2003"" xmlns:xsi=""http://www.w3.org/2001/XMLSchema-instance"" xmlns:xdUtil=""http://schemas.microsoft.com/office/infopath/2003/xslt/Util"" xmlns:xdXDocument=""http://schemas.microsoft.com/office/infopath/2003/xslt/xDocument"" xmlns:xdMath=""http://schemas.microsoft.com/office/infopath/2003/xslt/Math"" xmlns:xdDate=""http://schemas.microsoft.com/office/infopath/2003/xslt/Date"" xmlns:s0=""http://oakleaf.ws/nwordersws"" xmlns:dfs=""http://schemas.microsoft.com/office/infopath/2003/dataFormSolution"" xmlns:my=""http://schemas.microsoft.com/office/infopath/2003/myXSD/2003-11-26T19:52:29"" xmlns:mime=""http://schemas.xmlsoap.org/wsdl/mime/"" xmlns:tm=""http://microsoft.com/wsdl/mime/textMatching/"" xmlns:soapenc=""http://schemas.xmlsoap.org/soap/encoding/"" xmlns:soap=""http://schemas.xmlsoap.org/wsdl/soap/"" xmlns:http=""http://schemas.xmlsoap.org/wsdl/http/"" xmlns:ns1=""http://schemas.xmlsoap.org/wsdl/""")> _
	Public Class NWOrdersWSProject
		Private thisXDocument As XDocument
		Private thisApplication As Application
		'Change below to True to display a message box with the return values
		'of the running sum expression box
		Private blnShowMessage As Boolean = False
		Private strProdIDs As String
		Private strThisOrderID As String

		Public Sub _Startup(ByVal app As Application, ByVal doc As XDocument)
			thisXDocument = doc
			thisApplication = app
		End Sub

		Public Sub _Shutdown()
		End Sub

		<InfoPathEventHandler(EventType:=InfoPathEventType.OnLoad)> _
		 Public Sub OnLoad(ByVal e As DocReturnEvent)
			'Check all WebServiceAdapters (demonstration only)
			'The WebServiceAdapterObject and WebServiceAdapter2 interfaces are identical
			Dim colWSAdapters As DataAdapters = e.XDocument.DataAdapters
			Dim daWebService As WebServiceAdapterObject
			For Each daWebService In colWSAdapters
				'Inspection of new WebServiceAdapterObject properties
				With daWebService
					Dim strName As String = .Name
					Dim strInput As String = .Input
					Dim strOp As String = .Operation
					Dim strWSDL As String = .WSDLURL
					Dim nodOutputLoc As IXMLDOMNode = .OutputLocation
					Dim nodErrorsLoc As IXMLDOMNode = .ErrorsLocation
					Dim blnQuery As Boolean = .QueryAllowed
					Dim blnSubmit As Boolean = .SubmitAllowed
					Dim intTimeout As Integer = .Timeout
				End With
			Next
		End Sub

		<InfoPathEventHandler(MatchPath:="/dfs:myFields/dfs:dataFields/s0:GetOrderSPResponse/s0:GetOrderSPResult/s0:CustomerID", EventType:=InfoPathEventType.OnAfterChange)> _
		Public Sub CustomerID_OnAfterChange(ByVal e As DataDOMEvent)
			'Upper-case CustomerID value
			If e.IsUndoRedo Or e.Operation = "Delete" Then
				Return
			End If
			'Tests added for build 6058
			If Not TypeOf e.NewValue Is DBNull Then
				'Upper-case the text once (prevent reentry)
				If e.Source.text <> UCase(e.NewValue.ToString) Then
					e.Source.text = UCase(e.NewValue.ToString)
				End If
			End If
		End Sub

		'***********************************
		'Date and time tests and arithmetic
		'
		'1. Change Web service default null date value '0001-01-01T00:00:00...' to 
		'   client default '2001-01-01T00:00:00'. InfoPath doesn't recognize year
		'   0001 as a valid date and inserts the ISO 8601 text value.
		'2. If RequiredDate is default value, add 7 days to OrderDate to establish
		'   nominal required date (makes it easy to use the date picker to change it
		'3. Test for a RequiredDate less than five days after the OrderDate using
		'   the OnBeforeChange event handler
		'***********************************

		<InfoPathEventHandler(MatchPath:="/dfs:myFields/dfs:dataFields/s0:GetOrderSPResponse/s0:GetOrderSPResult/s0:RequiredDate", EventType:=InfoPathEventType.OnAfterChange)> _
		Public Sub RequiredDate_OnAfterChange(ByVal e As DataDOMEvent)
			'Fix null RequiredDate value returned by the Web service
			If e.IsUndoRedo Then
				Return
			End If
			Try
				Dim nodReqDate As IXMLDOMNode = _
				 e.Source.selectSingleNode("//s0:RequiredDate")
				If Not nodReqDate Is Nothing Then
					If Left(nodReqDate.text, 10) = "0001-01-01" Then
						nodReqDate.text = "2001-01-01T00:00:00"
					End If
				End If
			Catch excDate As Exception
				thisXDocument.UI.Alert(excDate.Message + vbCrLf + excDate.StackTrace)
			End Try
		End Sub

		<InfoPathEventHandler(MatchPath:="/dfs:myFields/dfs:dataFields/s0:GetOrderSPResponse/s0:GetOrderSPResult/s0:ShippedDate", EventType:=InfoPathEventType.OnAfterChange)> _
		Public Sub ShippedDate_OnAfterChange(ByVal e As DataDOMEvent)
			'Fix null ShippedDate value returned by Web service
			If e.IsUndoRedo Then
				Return
			End If
			Try
				Dim nodShipDate As IXMLDOMNode = _
				 e.Source.selectSingleNode("//s0:ShippedDate")
				If Not nodShipDate Is Nothing Then
					If Left(nodShipDate.text, 10) = "0001-01-01" Then
						nodShipDate.text = "2001-01-01T00:00:00"
					End If
				End If
			Catch excDate As Exception
				thisXDocument.UI.Alert(excDate.Message + vbCrLf + excDate.StackTrace)
			End Try
		End Sub

		<InfoPathEventHandler(EventType:=InfoPathEventType.OnContextChange)> _
		Public Sub OnContextChange(ByVal e As DocContextChangeEvent)
			'Add 7 days to OrderDate to change default ShipDate
			'Set the date after the OrderID is parsed on load
			If e.Type = "ContextNode" And e.Context.nodeName = "s0:intOrderID" Then
				Dim strContext As String = e.Context.nodeName
				Try
					'Fixup for hard-coded OrderDate value (replaces now())
					Dim nodOrderDate As IXMLDOMNode = _
					 e.XDocument.DOM.selectSingleNode("//s0:OrderDate")
					If Not nodOrderDate Is Nothing Then
						If nodOrderDate.text = "2001-01-01T00:00:00" Then
							'This only occurs on loading the empty form
							nodOrderDate.text = DateTime.Now.ToString("s")
						End If
					End If
					'Original code continues here
					Dim nodReqDate As IXMLDOMNode = _
					e.XDocument.DOM.selectSingleNode("//s0:RequiredDate")
					If Left(nodReqDate.text, 10) = "0001-01-01" Then
						'Fix null date from Web service, disregard time and time zone
						nodReqDate.text = "2001-01-01T00:00:00"
					End If
					Dim strReqDate As String = nodReqDate.text
					If nodReqDate.text = "2001-01-01T00:00:00" Then
						'Above is the default value
						Dim datReqDate As DateTime = Now
						datReqDate = datReqDate.AddDays(7)
						nodReqDate.text = datReqDate.ToString("s")
					End If
				Catch excDate As Exception
					thisXDocument.UI.Alert(excDate.Message + vbCrLf + excDate.StackTrace)
				End Try
			End If
		End Sub

		<InfoPathEventHandler(MatchPath:="/dfs:myFields/dfs:dataFields/s0:GetOrderSPResponse/s0:GetOrderSPResult/s0:RequiredDate", EventType:=InfoPathEventType.OnBeforeChange)> _
		Public Sub RequiredDate_OnBeforeChange(ByVal e As DataDOMEvent)
			'Test removed for builds > 6058
			Exit Sub
			'Test for a RequiredDate less than five days beyond the OrderDate
			Try
				Dim nodReqDate As IXMLDOMNode = _
				 e.Source.selectSingleNode("//s0:RequiredDate")
				If Not nodReqDate Is Nothing Then
					Dim strReqDate As String = nodReqDate.text
					'Test for default date/time values
					'Web service default includes the UTC time zone offset
					If Not (nodReqDate.text = "2001-01-01T00:00:00" Or _
					 Left(nodReqDate.text, 10) = "0001-01-01") Then
						Dim datRequired As DateTime = CType(strReqDate, DateTime)
						'Strip time component, because the RequiredDate time is 
						'later than OrderDate time
						datRequired = datRequired.Date
						Dim nodOrderDate As IXMLDOMNode = _
						 e.Source.selectSingleNode("//s0:OrderDate")
						If Not nodOrderDate Is Nothing Then
							'Test difference between RequiredDate and OrderDate
							Dim datOrder As DateTime = CType(nodOrderDate.text, DateTime)
							datOrder = datOrder.AddDays(5).Date
							Dim intCompare As Integer = _
							 DateTime.Compare(datOrder, datRequired)
							If intCompare > 0 Then
								Dim strMsg As String = "Required Date (" + _
								 datRequired.ToShortDateString + _
								 ") can't be less than five days after Order Date (" + _
								 datOrder.AddDays(-5).ToShortDateString + "). " + vbCrLf + _
								 "Required Date will revert to the original value."
								e.ReturnMessage = strMsg
								e.ReturnStatus = False
							Else
								e.ReturnStatus = True
							End If
						End If
					End If
				End If
			Catch excDate As Exception
				thisXDocument.UI.Alert(excDate.Message + vbCrLf + excDate.StackTrace)
			End Try
		End Sub

		'*********************************
		'Procedures for Order Details records
		'
		'1. Calculate running sum of extended value for each repeating row and
		'   display in table footer's Extended expression box. This procedure
		'   uses xdExtension in view1.xsl to call ExtendedTotal
		'2. Test for duplicate ProductID values (violates primary key constraint)
		'   with the OnBeforeChange event handler
		'3. Reset the test string for ProductID values with an order number change
		'3. Add the UnitPrice value for the selected product to the Unit Price text box
		'4. Limit the number of line items per order to 25 records with the 
		'   OnBeforeChange event handler
		'5. Fix a problem with missing OrderID values when adding the first new item
		'*********************************

		Public Function ExtendedTotal(ByVal objDetails As IXMLDOMNodeList) As String
			'Calculate running sum in expression box
			'This procedure returns the sum as a currency-formatted string ($10.00)
			'The procedure also generates a string for debugging
			Dim intNodes As Integer = objDetails.length
			Dim nodDetail As IXMLDOMNode
			Dim strXml As String			'for debugging
			Dim dblQuan As Double
			Dim dblPrice As Double
			Dim dblDisc As Double
			Dim dblExtended As Double
			Dim strExt As String
			Dim nodQuan As IXMLDOMNode
			Dim nodPrice As IXMLDOMNode
			Dim nodDisc As IXMLDOMNode
			Dim nodProdID As IXMLDOMNode
			Dim intNode As Integer			'for debugging
			Static Dim blnHasFired As Boolean
			Try
				For Each nodDetail In objDetails
					With nodDetail
						'In production code, this should be a loop with a select case
						'for the node name and text exiting at the first null value
						Dim strTemp As String = nodDetail.xml
						strTemp = Replace(strTemp, vbTab, "")
						strTemp = Replace(strTemp, vbCrLf, "")
						strXml += Replace(nodDetail.xml, vbTab, "") + vbCrLf
						nodQuan = .selectSingleNode("s0:Quantity")
						If Not nodQuan Is Nothing Then
							If IsNumeric(nodQuan.text) Then
								dblQuan = CType(nodQuan.text, Double)
							Else
								dblQuan = 0
							End If
						End If
						nodPrice = .selectSingleNode("s0:UnitPrice")
						If Not nodPrice Is Nothing Then
							If IsNumeric(nodPrice.text) Then
								dblPrice = CType(nodPrice.text, Double)
							Else
								dblPrice = 0
							End If
						End If
						nodDisc = .selectSingleNode("s0:Discount")
						If Not nodDisc Is Nothing Then
							If IsNumeric(nodDisc.text) Then
								dblDisc = CType(nodDisc.text, Double)
							Else
								dblDisc = 0
							End If
						End If
						nodProdID = .selectSingleNode("s0:ProductID")
						If Not nodProdID Is Nothing Then
							If IsNumeric(nodProdID.text) Then
								strProdIDs += nodProdID.text + " "
							End If
						End If
					End With
					'Following is for debugging with condition or alert box
					intNode = intNode + 1
					strExt += "Node " + intNode.ToString + ": Extended: " + Format(dblQuan * dblPrice * (1 - dblDisc), "$#,##0.00") + vbCrLf
					dblExtended += dblQuan * dblPrice * (1 - dblDisc)
				Next
				If dblExtended > 0 Then
					If blnShowMessage Then
						Dim strMsg As String
						'Uncomment to display XML with an Alert (not a Confirm)
						'strMsg = "XML: " + strXml + vbCrLf
						strMsg += strExt
						strMsg += "Return Value: " + Format(dblExtended, "$#,##0.00") + vbCrLf + vbCrLf
						strMsg += "Do you want to continue displaying this message?"
						'thisXDocument.UI.Alert(strMsg)
						If thisXDocument.UI.Confirm(strMsg, XdConfirmButtons.xdYesNo) = _
						 XdConfirmChoice.xdNo Then
							blnShowMessage = False
						End If
					End If
					Return Format(dblExtended, "$#,##0.00")
				Else
					'The else is for debugging
					Return "$0.00"
				End If
			Catch excExt As Exception
				thisXDocument.UI.Alert(excExt.Message + vbCrLf + excExt.StackTrace)
			End Try
		End Function

		<InfoPathEventHandler(MatchPath:="/dfs:myFields/dfs:dataFields/s0:GetOrderSPResponse/s0:GetOrderSPResult/s0:OrderDetails/s0:OrderDetail/s0:ProductID", EventType:=InfoPathEventType.OnBeforeChange)> _
		Public Sub ProductID_OnBeforeChange(ByVal e As DataDOMEvent)
			'Test for duplicate ProductID value (primary key constraint violation)
			Dim strProdID As String = ""
			Static strLastID As String = ""
			If e.IsUndoRedo Or e.Operation = "Delete" Then
				Return
			End If
			If IsNumeric(e.NewValue) Then
				If Val(e.NewValue.ToString) > 0 And Val(e.NewValue.ToString) < 78 Then
					Try
						'Test for duplicate ProductID values which violate composite
						'primary key constraint (OrderID + ProductID)
						strProdID = e.NewValue.ToString
						If strProdID <> strLastID And InStr(strProdIDs, strProdID) > 0 Then
							Dim strMsg As String = _
							 "You have already added Product ID " + _
							 strProdID + " to this order. " + _
							 "You can't add two line items for the same product."
							'If item value changed, not added, append the following
							If Not e.OldValue Is Nothing Then
								strMsg += vbCrLf + vbCrLf + _
								 "Product ID will revert to its original " + _
								 "value (" + e.OldValue.ToString + ")."
							End If
							e.ReturnMessage = strMsg
							e.ReturnStatus = False
						Else
							e.ReturnStatus = True
						End If
					Catch excProd As Exception
						thisXDocument.UI.Alert(excProd.Message + vbCrLf _
						 + excProd.StackTrace)
					End Try
				End If
			End If
		End Sub

		<InfoPathEventHandler(MatchPath:="/dfs:myFields/dfs:dataFields/s0:GetOrderSPResponse/s0:GetOrderSPResult/s0:OrderID", EventType:=InfoPathEventType.OnAfterChange)> _
		Public Sub GetOrderSPResult_OrderID_OnAfterChange(ByVal e As DataDOMEvent)
			If e.IsUndoRedo Or e.Operation = "Delete" Then
				Return
			End If
			'Reset the ProductID test string for each query or order
			strProdIDs = ""
		End Sub

		<InfoPathEventHandler(MatchPath:="/dfs:myFields/dfs:dataFields/s0:GetOrderSPResponse/s0:GetOrderSPResult/s0:OrderDetails/s0:OrderDetail/s0:ProductID", EventType:=InfoPathEventType.OnAfterChange)> _
		Public Sub ProductID_OnAfterChange(ByVal e As DataDOMEvent)
			'Add the UnitPrice value for the product to the Price text box
			If e.IsUndoRedo Or e.Operation = "Delete" Then
				Return
			End If
			Try
				Dim strProdID As String = e.NewValue.ToString
				Dim objProds As DataObject
				objProds = thisXDocument.DataObjects("GetProductsDetailed")
				Dim domProds As IXMLDOMDocument = objProds.DOM

				'The following three objects should be form-scope variables and
				'initialized/loaded in the single OnSwitchView event
				'This approach is for readability only
				Dim strXML As String = domProds.xml
				Dim docProds As New XmlDocument

				docProds.LoadXml(strXML)
				strXML = ""
				Dim lstProdID As XmlNodeList = _
				 docProds.GetElementsByTagName("prodID")
				Dim lstPrice As XmlNodeList = _
				 docProds.GetElementsByTagName("prodPrice")
				'For an "On Your Own" exercise (populate an expression box 
				'with the prodSKU value [QuantityPerUnit]}
				'Dim lstSKU As XmlNodeList = _
				' docProds.GetElementsByTagName("prodSKU")
				Dim intProds As Integer = lstProdID.Count
				Dim intCtr As Integer
				Dim nodProdID As XmlNode
				Dim blnFoundID As Boolean
				For intCtr = 0 To lstProdID.Count - 1
					'Find the element with the prodID value
					nodProdID = lstProdID.Item(intCtr)
					If nodProdID.InnerText = strProdID Then
						blnFoundID = True
						Exit For
					End If
				Next intCtr
				If blnFoundID Then
					Dim strPrice As String = lstPrice.Item(intCtr).InnerText
					'For an "On Your Own" exercise
					'Dim strSKU As String = lstSKU.Item(intCtr).InnerText
					'Get the grandparent of Source
					Dim nodRow As IXMLDOMNodeList = _
					 e.Source.parentNode.parentNode.selectNodes("*")
					If Not nodRow.Item(2) Is Nothing Then
						'Insert the UnitPrice
						nodRow.Item(2).text = strPrice
					End If
				Else
					'Add optional "Product not found" error message,
					'but should not occur with declarative restrictions
				End If
			Catch excProd As Exception
				thisXDocument.UI.Alert(excProd.Message + vbCrLf _
				 + excProd.StackTrace)
			End Try
		End Sub

		<InfoPathEventHandler(MatchPath:="/dfs:myFields/dfs:dataFields/s0:GetOrderSPResponse/s0:GetOrderSPResult/s0:OrderDetails/s0:OrderDetail", EventType:=InfoPathEventType.OnBeforeChange)> _
		Public Sub OrderDetail_OnBeforeChange(ByVal e As DataDOMEvent)
			'Limit the number of line items to 25
			If e.Operation = "Insert" And Not e.IsUndoRedo Then
				Dim strNode As String
				strNode = e.Source.parentNode.parentNode.nodeName
				Dim nodDetails As IXMLDOMNodeList = _
				 e.XDocument.DOM.selectNodes("//s0:OrderDetail")
				If Not nodDetails Is Nothing Then
					If nodDetails.length > 25 Then
						e.ReturnMessage = "The maximum number of line items " + _
						 "per order is 25. " + vbCrLf + vbCrLf + _
						 "This line item entry will be cancelled."
						e.ReturnStatus = False
					Else
						e.ReturnStatus = True
					End If
				Else
					e.ReturnStatus = True
				End If
			Else
				e.ReturnStatus = True
			End If
		End Sub

		'The following is a workaround for missing OrderID value on the
		'first insert to the OrderDetails table.
		<InfoPathEventHandler(MatchPath:="/dfs:myFields/dfs:dataFields/s0:GetOrderSPResponse/s0:GetOrderSPResult/s0:OrderDetails/s0:OrderDetail/s0:OrderID", EventType:=InfoPathEventType.OnAfterChange)> _
		Public Sub OrderDetail_OrderID_OnAfterChange(ByVal e As DataDOMEvent)
			If e.IsUndoRedo Or e.Operation = "Delete" Then
				Return
			End If
			Dim strOrderID As String = e.NewValue.ToString
			If strThisOrderID = "" And Val(e.Source.text) > 0 Then
				strThisOrderID = Val(e.Source.text).ToString
			End If
			If strOrderID = "" And strThisOrderID <> "" Then
				'Insert the OrderID value
				Dim nodOrderID As IXMLDOMNode = e.Source.selectSingleNode("s0:OrderID")
				'Test added for build 6058
				If Not nodOrderID Is Nothing Then
					nodOrderID.text = strThisOrderID
				End If
			End If
		End Sub

		'*********************************
		'Submit procedures for updating or inserting records
		'
		'1. Send the OrderID value to update a record; returns the OrderID
		'2. Send 0 as the OrderID value to insert a record; returns the new OrderID
		'
		'Note that InfoPath's Web service feature doesn't provide a
		'submit and receive data feature. Thus you must requery the GetLast10Orders
		'Web method to determine the inserted OrderID value. With a heavy workload,
		'there is the possibility that another user's insert might be returned
		'*********************************

		<InfoPathEventHandler(MatchPath:="btnUpdate", EventType:=InfoPathEventType.OnClick)> _
		Public Sub btnUpdate_OnClick(ByVal e As DocActionEvent)
			'Call the procedure for order updates
			Call UpdateOrInsertOrder(e, False)
		End Sub

		<InfoPathEventHandler(MatchPath:="btnInsert", EventType:=InfoPathEventType.OnClick)> _
		Public Sub btnInsert_OnClick(ByVal e As DocActionEvent)
			'Call the procedure to insert a new order
			Call UpdateOrInsertOrder(e, True)
		End Sub

		Private Sub UpdateOrInsertOrder(ByVal e As DocActionEvent, ByVal blnInsert As Boolean)
			'Procedure updates an order with a valid OrderID
			'Inserts a new order with 0 as the OrderID value
			Try
				Dim strOrderID As String
				Dim nodOrderId As IXMLDOMNode = e.Source.selectSingleNode("//s0:OrderID")
				strOrderID = nodOrderId.text
				If blnInsert Then
					'Required to insert and order
					nodOrderId.text = "0"
				End If
				Dim colWSAdapters As DataAdapters = e.XDocument.DataAdapters
				Dim daOrder As WebServiceAdapter2
				'Requires a type assignment
				daOrder = CType(colWSAdapters("UpdateOrInsertOrderSP"), _
				  WebServiceAdapter2)
				If Not daOrder Is Nothing Then
					'Demo SubmitAllowed
					If daOrder.SubmitAllowed Then
						daOrder.Submit()
					End If
				End If
				If blnInsert Then
					'Requery the GetLast10Orders Web method
					Dim daLast10 As WebServiceAdapter2
					daLast10 = CType(colWSAdapters("Last10Orders"), WebServiceAdapter2)
					If daLast10.QueryAllowed Then
						'Refresh the list
						daLast10.Query()
						If Not daLast10 Is Nothing Then
							Dim lstOrders As IXMLDOMNodeList
							lstOrders = daLast10.OutputLocation.selectNodes("//*")
							If Not lstOrders Is Nothing Then
								'The seventh node of 16 has the last OrderID value
								strOrderID = lstOrders.Item(6).text
								nodOrderId.text = strOrderID
							End If
						End If
					End If
				End If
				'Display a confirmation message
				Dim strMsg As String = "Order " + strOrderID + " successfully "
				If blnInsert Then
					strMsg += "inserted "
				Else
					strMsg += "updated "
				End If
				strMsg += Now.ToLongDateString + " at " + Now.ToLongTimeString + _
				 " by the " + daOrder.Name + _
				 " Web method of the NWOrdersWS Web service at " + _
				 daOrder.WSDLURL + "."
				thisXDocument.UI.Alert(strMsg)
			Catch excOrder As Exception
				'Extended version for debugging
				'thisXDocument.UI.Alert(excOrder.Message + vbCrLf + excOrder.StackTrace)
				thisXDocument.UI.Alert(excOrder.Message)
			End Try
		End Sub
	End Class
End Namespace
