﻿Option Explicit On 
Option Strict On
Imports System
Imports Microsoft.Office.Interop.InfoPath.SemiTrust
Imports Microsoft.VisualBasic
'Required for validation
Imports System.Xml
Imports System.Xml.Schema
Imports System.IO
'Required for StringBuilder
Imports System.Text
'Required for machine name
Imports System.Environment

' Office integration attribute. Identifies the startup class for the form. Do not
' modify.
<Assembly: System.ComponentModel.DescriptionAttribute("InfoPathStartupClass, Version=1.0, Class=Rss2Production.Rss2Production")>

Namespace Rss2Production
	' The namespace prefixes defined in this attribute must remain synchronized with
	' those in the form definition file (.xsf).
	<InfoPathNamespace("xmlns:xdUtil=""http://schemas.microsoft.com/office/infopath/2003/xslt/Util"" xmlns:xdXDocument=""http://schemas.microsoft.com/office/infopath/2003/xslt/xDocument"" xmlns:xhtml=""http://www.w3.org/1999/xhtml"" xmlns:xd=""http://schemas.microsoft.com/office/infopath/2003"" xmlns:my=""http://schemas.microsoft.com/office/infopath/2003/myXSD/2003-12-10T19-37-13"" xmlns:msxsl=""urn:schemas-microsoft-com:xslt"" xmlns:xdMath=""http://schemas.microsoft.com/office/infopath/2003/xslt/Math"" xmlns:xsf=""http://schemas.microsoft.com/office/infopath/2003/solutionDefinition"" xmlns:xdDate=""http://schemas.microsoft.com/office/infopath/2003/xslt/Date"" xmlns:xsi=""http://www.w3.org/2001/XMLSchema-instance""")> _
	Public Class Rss2Production
		Private thisXDocument As XDocument
		Private thisApplication As Application
		Private strViewName As String = " (Items Only)"
		Private strFileName As String
		Private strFilePath As String
		Private strRssPath As String
		Private strErrors As String
		'Change value to True to display network and user information alert
		Private blnDisplayStatusMessage As Boolean = False

		Public Sub _Startup(ByVal app As Application, ByVal doc As XDocument)
			thisXDocument = doc
			thisApplication = app
		End Sub

		Public Sub _Shutdown()
		End Sub

		<InfoPathEventHandler(EventType:=InfoPathEventType.OnLoad)> _
		Public Sub OnLoad(ByVal e As DocReturnEvent)
			Try
				If blnDisplayStatusMessage Then
					'Application properties and methods examples
					'Change the values to suit your network setup
					Dim strMsg As String = "Testing " + MachineName + _
					"'s network connectivity, user name, " + _
					"group membership, and InfoPath version." + vbCrLf + vbCrLf
					'The user info is provided by System.Environment
					Dim strLogin As String = UserDomainName + "\" + UserName
					With thisApplication
						'Check network connectivity
						Dim blnOnline As Boolean
						If .MachineOnlineState = XdMachineOnlineState.xdOnline Then
							blnOnline = True
							'The MachineName is provided by System.Environment
							strMsg += MachineName + " is on line." + vbCrLf
							'Ping for network resource availability
							strMsg += "\\Oakleaf-DC1\Backup file share is "
							Dim blnLAN As Boolean = _
							 .IsDestinationReachable("\\OakLeaf-DC1\Backup")
							If Not blnLAN Then
								strMsg += "not "
							End If
							strMsg += "reachable." + vbCrLf + _
							 "Web server http://www.oakleaf.ws is "
							Dim blnWeb As Boolean = _
							 .IsDestinationReachable("http://www.oakleaf.ws")
							If Not blnWeb Then
								strMsg += "not "
							End If
							strMsg += "reachable." + vbCrLf
						Else
							'Warn user and disable operations that require 
							'network(connectivity)
							strMsg = MachineName + " is off line." + vbCrLf
						End If

						'Check user identity and group membership to suit your setup
						'The group tests require a fully-trusted form
						Dim strUser As String = strLogin + " is "
						Dim strMemb As String = "a member of "
						Dim strNot As String = "not "
						With .User
							strMsg += strUser
							Dim blnIsUserAdmin As Boolean = _
							 .IsCurrentUser("OAKLEAF\Administrator")
							If Not blnIsUserAdmin Then
								strMsg += strNot
							End If
							strMsg += "OAKLEAF\Administrator" + vbCrLf + strUser
							Dim blnIsGroupDomAdmins As Boolean = _
							 .IsUserMemberOf("OAKLEAF\Domain Admins")
							If Not blnIsGroupDomAdmins Then
								strMsg += strNot
							End If
							strMsg += strMemb + "Domain Admins." + vbCrLf + strUser
							Dim blnIsGroupIPDesigners As Boolean = _
							 .IsUserMemberOf("OAKLEAF\InfoPathDesigners")
							If Not blnIsGroupIPDesigners Then
								strMsg += strNot
							End If
							strMsg += strMemb + "InfoPathDesigners." + vbCrLf + strUser
							Dim blnIsGroupIPUsers As Boolean = _
							 .IsUserMemberOf("OAKLEAF\InfoPathUsers")
							If Not blnIsGroupIPUsers Then
								strMsg += strNot
							End If
							strMsg += strMemb + "InfoPathUsers." + vbCrLf + strUser
							'The following tests Distribution (not Security) Groups
							Dim blnIsGroupIPDesignersDist As Boolean = _
							 .IsUserMemberOf("OAKLEAF\IPDesignersDist")
							If Not blnIsGroupIPDesignersDist Then
								strMsg += strNot
							End If
							strMsg += strMemb + "IPDesignersDist (Distribution Group)." + _
							 vbCrLf + strUser
							Dim blnIsGroupIPUsersDist As Boolean = _
							 .IsUserMemberOf("OAKLEAF\IPUsersDist")
							If Not blnIsGroupIPUsersDist Then
								strMsg += strNot
							End If
							strMsg += strMemb + "IPDesignersDist (Distribution Group)." + _
							 vbCrLf
							'Logic to restrict user activities based on user name
							'and/or group membership goes here
						End With
						'Return the InfoPath majorversion.minorversion.build
						strMsg += "Current InfoPath 2003 version is: " + .Version
						thisXDocument.UI.Alert(strMsg)
					End With
				End If

				'Set default path (template folder) and file name
				'Get the path from the Solution.URI property
				Dim strPath As String = e.XDocument.Solution.URI
				Dim blnIsWebSite As Boolean				  'Web and SharePoint sites
				Dim blnIsWSS As Boolean				 'SharePoint site
				Dim blnIsFile As Boolean
				Dim strTestFile As String = "file:///"
				Dim strTestHTTP As String = "http://"
				Dim strTestWSS As String = "/Forms/"
				'Set the flags
				If InStr(strPath, strTestFile) > 0 Then
					blnIsFile = True
				ElseIf InStr(strPath, strTestHTTP) > 0 Then
					blnIsWebSite = True
					If InStr(strPath, strTestWSS) > 0 Then
						blnIsWSS = True
					End If
				End If

				If blnIsFile Or blnIsWebSite Then
					'Trim the URI for the path to remove the template file
					If blnIsWebSite Then
						strPath = Mid(strPath, InStr(strPath, strTestHTTP))
						strPath = Left(strPath, InStrRev(strPath, "/") - 1)
					Else
						'Remove the prefix
						strPath = Mid(strPath, InStr(strPath, strTestFile) + _
						 Len(strTestFile))
						strPath = Left(strPath, InStrRev(strPath, "\") - 1)
						'Replace URL-encoded spaces
						strPath = Replace(strPath, "%20", " ")
						strFilePath = strPath
					End If
				End If
				Dim datFile As DateTime = Now
				Dim strFile As String = datFile.ToString("s")
				'Remove seconds, hyphens and colons
				strFile = "Rss" + Left(strFile, InStrRev(strFile, ":") - 1) + ".xml"
				strFile = Replace(strFile, ":", "")
				strFile = Replace(strFile, "-", "")
				strFileName = strFile
				'Set the dialog variables
				With e.XDocument.UI
					If Not blnIsWSS Then
						'SharePoint determines the path
						.SetSaveAsDialogLocation(strPath)
					End If
					.SetSaveAsDialogFileName(strFile)

					'Temporary alert for testing templates published to
					'Web and SharePoint sites
					Dim strSaveAs As String = _
					 "Solution.URI = '" + e.XDocument.Solution.URI + "'" + vbCrLf
					If blnIsWSS Then
						strSaveAs += "Form file = '" + strFile + "'"
					ElseIf blnIsWebSite Then
						strSaveAs += "Form server, directory, and file = '" + _
						 strPath + "/" + strFile + "'"
					Else
						strSaveAs += "Form path and file = '" + strPath + "\" + _
						 strFile + "'"
					End If
					'.Alert(strSaveAs)
				End With

				'Get all pubDate elements
				Dim lstItems As IXMLDOMNodeList = e.XDocument.DOM.selectNodes("//pubDate")
				'Check all pubDate nodes
				Dim intItem As Integer
				For intItem = 0 To lstItems.length - 1
					Dim nodLastItem As IXMLDOMNode = lstItems(intItem)
					nodLastItem.selectSingleNode("//pubDate")
					If nodLastItem.text = "" Then
						'Get current system time as UTC if pubDate is empty
						Dim datNow As DateTime = DateTime.UtcNow
						'Add the RFC1123 date
						nodLastItem.text = datNow.ToString("R")
					End If
				Next intItem
				'Don't treat adding the default date as a document change
				thisXDocument.SetDirty(False)
			Catch excLoad As Exception
				If InStr(excLoad.Message, "Item has already been added") = 0 Then
					'Don't display these errors (workaround)
					thisXDocument.UI.Alert(excLoad.Message + excLoad.StackTrace)
				End If
			End Try
		End Sub

		'*********************************************
		'Document creation, rss-no_body.xml validation
		'and rss.xml file save procedures with form
		'and custom toolbar buttons, plus error checking
		'for the Save rss.xml File toolbar buttons
		'*********************************************

		<InfoPathEventHandler(MatchPath:="btnSaveRSS2", EventType:=InfoPathEventType.OnClick)> _
		Public Sub btnSaveRSS2_OnClick(ByVal e As DocActionEvent)
			'Form button on ItemsOnly view
			Call SaveRssToFile()
		End Sub

		<InfoPathEventHandler(MatchPath:="btnSaveRSS", EventType:=InfoPathEventType.OnClick)> _
		Public Sub btnSaveRSS_OnClick(ByVal e As DocActionEvent)
			'Form button on AllSections view
			Call SaveRssToFile()
		End Sub

		<InfoPathEventHandler(MatchPath:="btnSaveRssFileTB1", EventType:=InfoPathEventType.OnClick)> _
		Public Sub btnSaveRssFileTB1_OnClick(ByVal e As DocActionEvent)
			'Toolbar button on AllSections view
			If TestFormForErrors() Then
				Call SaveRssToFile()
			End If
		End Sub

		Private Function TestFormForErrors() As Boolean
			'Test for form errors: Return True if none, False if errors exist
			'This test is required for toolbar buttons only
			Try
				Dim intErrs As Integer
				Dim colErrors As ErrorsCollection = thisXDocument.Errors
				If colErrors.Count = 0 Then
					'Form is OK
					Return True
				Else
					'Display a detailed error alert
					Dim strMsg As String
					Dim intErr As Integer
					With colErrors
						strMsg = "You can't save the rss.xml file because the form has " + .Count.ToString + " errors:" + vbCrLf + vbCrLf
						'Loop runs in reverse due to the sequence of addition
						'of the validation rules for the fields
						For intErr = .Count - 1 To 0 Step -1
							strMsg += "Error " + (.Count - intErr).ToString + ": "
							strMsg += .Item(intErr).ShortErrorMessage + vbCrLf
						Next intErr
						thisXDocument.UI.Alert(strMsg)
					End With
					Return False
				End If
			Catch excErr As Exception
				thisXDocument.UI.Alert(excErr.Message + excErr.StackTrace)
			End Try
		End Function

		Private Sub SaveRssToFile()
			'Modify and format the InfoPath document, then save it to an rss.xml file
			Dim strXMLNS As String = " xmlns=""http://www.w3.org/1999/xhtml"""
			Dim strXML As String = "<?xml version=""1.0"" encoding=""UTF-8""?>"
			Dim strLang As String = " xml:lang=""en-us"""
			Dim strFile As String

			'Hard-code the file path for rss.xml and rss-no_body.xml
			'The production version writes directly to the OakLeaf Web server
			strRssPath = "C:\InfoPathSP-1\Rss2Production"

			Try
				Dim strRSS As String = thisXDocument.DOM.xml
				'Remove whitespace
				'ControlChars.* is a generic alternative to vbTab and vbCrLf)
				strRSS = Replace(strRSS, ControlChars.Tab, "")
				strRSS = Replace(strRSS, ControlChars.CrLf, "")
				'Remove the PIs
				strRSS = Mid(strRSS, InStr(strRSS, "<rss"))
				'Add the XML declaration
				strRSS = strXML + strRSS
				'Remove the xml:lang="en-us" attribute
				strRSS = Replace(strRSS, strLang, "")
				'Remove empty body elements,if present
				strRSS = Replace(strRSS, "<body/>", "")
				'Remove xmlns="http://www.w3.org/1999/xhtml" attributes in body text
				strRSS = Replace(strRSS, strXMLNS, "")
				'Add the namespace attribute to the body elements (RSS 2.0 convention)
				strRSS = Replace(strRSS, "<body>", "<body " + strXMLNS + ">")

				Dim strDoc As String
				Dim intFile As Integer
				Dim blnNoBody As Boolean
				'Following code indents two versions of the document 
				'with an XmlTextReader because XHTML won't validate
				'If the rss-no_body.xml file doesn't validate, rss.xml isn't created
				'Note that it requires a lot of code just to add indents
				For intFile = 0 To 1
					If intFile = 0 Then
						'Exclude <body> elements on first pass for validation
						blnNoBody = True
					Else
						'Include <body> elements on second pass if validated
						blnNoBody = False
					End If
					'Use a StringBuilder to append the elements and text
					Dim sbDoc As New StringBuilder
					'Create a NameTable and XmlNamespaceManager using the
					'example from online help (overkill for no namespace)
					Dim nt As NameTable = New NameTable
					Dim nsmgr As XmlNamespaceManager = New XmlNamespaceManager(nt)
					'Create an XmlParserContext
					Dim context As XmlParserContext = New XmlParserContext(Nothing, nsmgr, Nothing, XmlSpace.None)
					'Create an XmlTextReader 
					Dim xtrRSS As XmlTextReader = New XmlTextReader(strRSS, XmlNodeType.Element, context)
					With xtrRSS
						Dim strName As String
						Dim strValue As String
						Dim blnIsBody As Boolean
						'Read each element, attribute, and value
						Do While xtrRSS.Read
							Select Case .NodeType
								Case XmlNodeType.Element
									'Add an element and its attribute, if present
									If Not blnIsBody Then
										'Don't add spaces to body XHTML sub-elements
										sbDoc.Append(Space(.Depth * 2))
									End If
									If .Name = "body" Then
										blnIsBody = True
									End If
									If Not (blnNoBody And blnIsBody) Then
										sbDoc.Append("<" + .Name)
										'Required because of attribute test
										strName = .Name
										If .MoveToFirstAttribute Then
											'Only one attribute per element in this document
											sbDoc.Append(" " + .Name + "=""" + .Value + """>")
										Else
											sbDoc.Append(">")
										End If
									End If
									If InStr("rss channel image item", strName) > 0 And _
									 Len(strName) > 2 Then
										sbDoc.Append(vbCrLf)
									End If
								Case XmlNodeType.Text
									If Not (blnNoBody And blnIsBody) Then
										'Add the element's text
										strValue = .Value
										'Change illegal characters to entities
										strValue = Replace(strValue, "&", "&amp;")
										strValue = Replace(strValue, "<", "&lt;")
										strValue = Replace(strValue, ">", "&gt;")
										sbDoc.Append(strValue)
									End If
								Case XmlNodeType.EndElement
									'Add the closing element
									If InStr("rss channel image item", .Name) > 0 And _
									 Len(.Name) > 2 Then
										'Add indent if not a body element
										sbDoc.Append(Space(.Depth * 2))
									End If
									If Not (blnNoBody And blnIsBody) Then
										sbDoc.Append("</" + .Name + ">")
									End If
									strName = .Name
									If .Name = "body" Then
										blnIsBody = False
									End If
									If Not blnIsBody Then
										sbDoc.Append(vbCrLf)
									End If
							End Select
						Loop
					End With
					'Cast StringBuilder as String
					strDoc = sbDoc.ToString

					'Remove empty body (one space), if present
					strDoc = Replace(strDoc, _
					 "      <body" + strXMLNS + "> </body>" + vbCrLf, "")

					'Test for well-formedness (throws an exception if malformed)
					Dim docTest As New XmlDocument
					docTest.LoadXml(strDoc)

					If blnNoBody Then
						'Validate the NoBody version
						Call ValidateNoBodyDocument(strDoc)
						If strErrors <> "" Then
							'Validation failed
							thisXDocument.UI.Alert("Validation error(s): " + _
							 strErrors + vbCrLf + _
							 "You can't save a document that fails validation.")
							'Depart
							Exit For
						End If
					End If

					'Save the file
					If intFile = 1 Then
						strFile = strRssPath + "\rss.xml"
					Else
						strFile = strRssPath + "\rss-no_body.xml"
					End If
					Dim filRSS As File
					Dim blnWriteFile As Boolean
					If filRSS.Exists(strFile) Then
						'Get the file date and time
						Dim strDate As String = _
						 filRSS.GetLastWriteTime(strFile).ToLongDateString
						Dim strTime As String = _
						 filRSS.GetLastWriteTime(strFile).ToLongTimeString
						'Construct the question
						Dim strMsg As String = "This file will overwrite '" + _
						 strFile + "', which was last updated on " + strDate + _
						 " at " + strTime + ". Do you want to proceed?"
						If intFile = 1 Then
							'Only ask once, always write the no-body version
							If thisXDocument.UI.Confirm(strMsg, _
							 XdConfirmButtons.xdYesNo) = XdConfirmChoice.xdYes Then
								blnWriteFile = True
							End If
						Else
							blnWriteFile = True
						End If
					Else
						blnWriteFile = True
					End If
					If blnWriteFile Then
						'Write the file to disk (requires fully-trusted code/form)
						Dim stwRSS As StreamWriter = _
						 New StreamWriter(strFile, False, Encoding.UTF8)
						stwRSS.Write(strDoc)
						stwRSS.Flush()
						stwRSS.Close()
					End If
				Next intFile
			Catch excRSS As Exception
				thisXDocument.UI.Alert(excRSS.Message + excRSS.StackTrace)
			End Try
		End Sub

		Private Sub ValidateNoBodyDocument(ByVal strDoc As String)
			'Validate the no <body> version against the RSS 2.0 schema
			Dim strRssSchema As String = strRssPath + "\rss-2_0.xsd"
			If File.Exists(strRssSchema) Then
				strErrors = ""
				'Create a schema collection and add the rss-2_0.xsd schema
				Dim xscRSS As New XmlSchemaCollection
				xscRSS.Add("", strRssSchema)
				'Create a StringReader, XmlTextReader, and XmlValidatingReader
				Dim srRSS As New StringReader(strDoc)
				Dim xtrRSS As New XmlTextReader(srRSS)
				Dim xvrRSS As New XmlValidatingReader(xtrRSS)
				With xvrRSS
					'Specify the validation type (XSD) and add the schema
					.ValidationType = ValidationType.Schema
					.Schemas.Add(xscRSS)
					'Specify a ValidationError event handler
					AddHandler .ValidationEventHandler, _
					 AddressOf ValidationError
					While .Read
						'Process the document
					End While
					.Close()
				End With
			Else
				strErrors = "The schema file ('" + strRssSchema + "') is missing. Can't validate the document."
			End If
		End Sub

		Private Sub ValidationError(ByVal sender As Object, _
		 ByVal args As ValidationEventArgs)
			'Event handler for intercepting validation errors
			strErrors += args.Message.ToString + vbCrLf
		End Sub

		'******************************************************
		'OnAfterChange event handlers to add RFC1123
		'formatted date and copy text from one field to another
		'******************************************************

		<InfoPathEventHandler(MatchPath:="/rss/channel/item/title", EventType:=InfoPathEventType.OnAfterChange)> _
		Public Sub item_title_OnAfterChange(ByVal e As DataDOMEvent)
			'Add current GMT values in RFC1123 format
			If e.IsUndoRedo Or e.Operation = "Delete" Or e.Site.nodeName <> "title" Then
				Return
			End If
			Try
				Dim nodLastItem As IXMLDOMNode = e.Source.parentNode.selectSingleNode("//pubDate")
				If nodLastItem.text = "" Then
					'Get current system time as UTC if pubDate is empty
					Dim datNow As DateTime = DateTime.UtcNow
					'Add the RFC1123 date
					nodLastItem.text = datNow.ToString("R")
				End If
			Catch excInsert As Exception
				thisXDocument.UI.Alert(excInsert.Message + excInsert.StackTrace)
			End Try
		End Sub

		<InfoPathEventHandler(MatchPath:="/rss/channel/item/description", EventType:=InfoPathEventType.OnAfterChange)> _
		Public Sub item_description_OnAfterChange(ByVal e As DataDOMEvent)
			'Copy description to body if body is empty
			If e.IsUndoRedo Or e.Operation = "Delete" Then
				Return
			End If
			Dim strDescr As String = e.Source.nodeValue.ToString
			If strDescr <> "" Then
				'Update the body element
				Dim nodBody As IXMLDOMNode = _
				 e.Source.parentNode.selectSingleNode("//body")
				If nodBody.text = "" Then
					nodBody.text = strDescr
				End If
				Exit Sub
			End If
		End Sub

		<InfoPathEventHandler(MatchPath:="/rss/channel/item/link", EventType:=InfoPathEventType.OnAfterChange)> _
  Public Sub item_link_OnAfterChange(ByVal e As DataDOMEvent)
			'Copy the link value to guid, if guid hasn't been updated
			If e.IsUndoRedo Or e.Operation = "Delete" Then
				Return
			End If
			Dim strLink As String = e.Source.nodeValue.ToString
			If strLink <> "http://www.oakleaf.ws/" And strLink <> "" Then
				'Update the guid element
				Dim nodGuid As IXMLDOMNode = _
				 e.Source.parentNode.selectSingleNode("//guid")
				If nodGuid.text = "http://www.oakleaf.ws/" Then
					nodGuid.text = strLink
				End If
			End If
		End Sub

		'**********************************************
		'View event handlers added in previous chapters
		'Added toolbar button on AllSections view
		'**********************************************

		<InfoPathEventHandler(MatchPath:="btnItemsOnlyTB1", EventType:=InfoPathEventType.OnClick)> _
		 Public Sub btnItemsOnlyTB1_OnClick(ByVal e As DocActionEvent)
			'Toolbar button on AllSections view
			strViewName = " (Items Only)"
			thisXDocument.View.SwitchView("ItemsOnly")
		End Sub

		<InfoPathEventHandler(MatchPath:="btnViewAll", EventType:=InfoPathEventType.OnClick)> _
		Public Sub btnViewAll_OnClick(ByVal e As DocActionEvent)
			Call DisplayMultipleFormMessage()
			'Pass the view name to the new view
			strViewName = " (All Sections)"
			thisXDocument.View.SwitchView("AllSections")
		End Sub

		Private Sub DisplayMultipleFormMessage()
			With thisXDocument.View.Window
				Try
					'Display an informative alert if more than one form is open
					Dim intNumXDocs As Integer = thisApplication.XDocuments.Count
					Dim strXDocNames As String
					Dim strName As String = .Caption
					Dim strType As String = .WindowType.ToString
					Dim strState As String = .WindowState.ToString
					Dim intCtr As Integer
					For intCtr = 0 To intNumXDocs - 1
						'Get the open form's caption in the title bar
						strXDocNames += thisApplication.XDocuments.Item(intCtr).View.Window.Caption + ", "
					Next intCtr
					'Trim the names string
					strXDocNames = Left(strXDocNames, Len(strXDocNames) - 2)
					If intNumXDocs > 1 Then
						'Display details
						thisXDocument.UI.Alert("There are " + intNumXDocs.ToString + _
						" XDocuments in the collection: " + strXDocNames + vbCrLf + _
						"Active Window: " + strName + vbCrLf + "Window Type: " + _
						strType + vbCrLf + "Window State: " + strState)
					End If
				Catch
					'Disregard errors
				End Try
			End With
		End Sub

		<InfoPathEventHandler(MatchPath:="btnViewItems", EventType:=InfoPathEventType.OnClick)> _
		Public Sub btnViewItems_OnClick(ByVal e As DocActionEvent)
			'Form button on AllSections view
			strViewName = " (Items Only)"
			thisXDocument.View.SwitchView("ItemsOnly")
		End Sub

		<InfoPathEventHandler(EventType:=InfoPathEventType.OnSwitchView)> _
		Public Sub OnSwitchView(ByVal e As DocEvent)
			Try
				With thisXDocument.View.Window
					If (Left(.Caption, 4) = "Form" And Val(Mid(.Caption, 5, 1)) > 0) _
					 Or Left(.Caption, 7) = "Preview" Then
						'Replace caption if Form#[#] or Preview*
						.Caption = strFileName
					End If
					'Force normal window state
					.WindowState = XdWindowState.xdWindowStateNormal
					'Set fixed dimensions for width only
					'.Height = 380
					.Width = 530
					If thisApplication.UsableHeight >= .Height And _
					 thisApplication.UsableWidth >= .Width Then
						'Center the form horizontally
						'.Top = (thisApplication.UsableHeight - .Height) \ 2
						.Left = (thisApplication.UsableWidth - .Width) \ 2
					End If
					'Remove previous view name
					If InStr(.Caption, " (") > 0 Then
						.Caption = Left(.Caption, InStr(.Caption, " (") - 1)
					End If
					'Add the new view name
					.Caption = .Caption + strViewName
				End With
			Catch excView As Exception
				thisXDocument.UI.Alert(excView.Message + excView.StackTrace)
			End Try
		End Sub
	End Class
End Namespace
